/*******************************************************************
 *
 * Licensed Materials - Property of IBM
 * 
 * AJAX Toolkit Framework 6-28-496-8128
 * 
 * (c) Copyright IBM Corp. 2006 All Rights Reserved.
 * 
 * U.S. Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 *******************************************************************/

package org.eclipse.atf.ui.debug;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.atf.project.FlexibleProjectUtils;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.dialogs.ElementTreeSelectionDialog;
import org.eclipse.ui.model.BaseWorkbenchContentProvider;

/**
 * @author sedota
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class ApplicationFileSelectionDialog	extends ElementTreeSelectionDialog {

	protected String[] fExtensions = null;
	protected String[] fRequiredNatures = null;
	
	protected static BaseWorkbenchContentProvider cp = new BaseWorkbenchContentProvider() {
	
		protected Map wcfs = new HashMap();
		
		final class WebContentFolder {
			private IContainer webContent;
			private IFolder folder;
			private IProject curr;
			private int segment;
			
			public WebContentFolder( IProject project, IContainer webContent ) {
				this.webContent = webContent;
				this.segment = 0;
				//this.curr = webContent.getProjectRelativePath().segment(segment);
				this.curr = project;
				//segment++;
			}
			
			public boolean hasNext() {
				return segment < webContent.getProjectRelativePath().segmentCount();
			}
			
			public IFolder next() {
				String folderName = webContent.getProjectRelativePath().segment(segment);
				if( segment != 0 ) {
					folder = folder.getFolder(folderName);
				} else {
					folder = curr.getFolder(folderName);
				}
				segment++;
				return folder;
			}
		}
		
		public boolean hasChildren(Object element) {
			if( element instanceof IProject ) {
				IContainer webContent = FlexibleProjectUtils.getWebContentFolder((IProject)element);
				if( element == webContent || webContent != null ) {
					return true;
				}
			} else if( element instanceof IFolder ) {
				try {
					return ((IFolder)element).members().length > 0;
				} catch (CoreException e) {}
			}
			return false;
		}
	
		public Object[] getElements(Object element) {
			IProject[] projects = (IProject[])element;
			List webContentProjects = new ArrayList();
			for( int i = 0; i < projects.length; i++ ) {
				IContainer webContent = FlexibleProjectUtils.getWebContentFolder(projects[i]);
				if( webContent != null ) {
					webContentProjects.add(projects[i]);
				}
			}
			return webContentProjects.toArray();
		}
	
		public Object[] getChildren(Object element) {
			if( element instanceof IProject ) {
				IContainer webContent = FlexibleProjectUtils.getWebContentFolder((IProject)element);
				if( element == webContent ) {
					try {
						return ((IProject)element).members();
					} catch (CoreException e) {}
				} else if( webContent != null ) {
					WebContentFolder wcf = new WebContentFolder((IProject)element, webContent);
					wcfs.put(element, wcf);
					if( wcf.hasNext() ) {
						return new Object[]{wcf.next()};
					}
				}
			} else if( element instanceof IFolder ) {
				WebContentFolder wcf = (WebContentFolder)wcfs.get(((IFolder)element).getProject());
				if( wcf == null ) {
					return super.getChildren(element);
				}
				if( wcf.hasNext() ) {
					return new Object[]{wcf.next()};
				} else {
					try {
						return ((IFolder)element).members();
					} catch (CoreException e) {}
				}
			}
			return new Object[0];
		}
	
	};
	
	/**
	 * ApplicationFileSelectionDialog constructor comment.
	 * @param parent Shell
	 * @parent extensions String[]
	 */
	/*public ApplicationFileSelectionDialog(Shell parent, String[] extensions, String[] natureIds) {
		this(parent, null, null, extensions, false);
		fRequiredNatures = natureIds;
	}*/
	
	/**
	 * FilteredFileSelectionDialog constructor comment.
	 * @param parent Shell
	 * @param title String
	 * @param message String
	 * @parent extensions String[]
	 * @param allowMultiple boolean
	 */
	/*public ApplicationFileSelectionDialog(Shell parent, String title, String message, String[] extensions, boolean allowMultiple) {
		super(parent, new WorkbenchLabelProvider(), new BaseWorkbenchContentProvider());
		setShellStyle(SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL | SWT.RESIZE);
	
		setTitle(title);	
		if(title == null) setTitle("Title"); //$NON-NLS-1$
		if(message == null) message = "Message"; //$NON-NLS-1$
		setMessage(message);
		setAllowMultiple(allowMultiple);
		setExtensions(extensions);
		addFilter( new ApplicationFileViewerFilter(fRequiredNatures, fExtensions));
		
		setValidator( new TypedElementSelectionValidator(new Class[] {IFile.class}, allowMultiple) );
		
	}*/
		
	/**
	 * FilteredFileSelectionDialog constructor comment.
	 * @param parent Shell
	 * @param title String
	 * @param message String
	 * @parent extensions String[]
	 * @param allowMultiple boolean
	 */
	public ApplicationFileSelectionDialog(Shell parent, ILabelProvider labelProvider, String title, String message, IProject[] projects, String[] requiredNatures, boolean allowMultiple) {
		super(parent, labelProvider, cp );
		
		fRequiredNatures = requiredNatures;
		setShellStyle(SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL | SWT.RESIZE);
		setTitle(title);	
		if(title == null) setTitle("Title"); //$NON-NLS-1$
		if(message == null) message = "Message"; //$NON-NLS-1$
		setMessage(message);
		setAllowMultiple(allowMultiple);
		setValidator( new TypedElementSelectionValidator(new Class[] {IFile.class}, allowMultiple) );
		
	}
	
	public String[] getExtensions() {
		return fExtensions;
	}
	public void setExtensions(String[] extensions) {
		fExtensions = extensions;
	}
			
}
